%% ~*Efficient Bounding Method*~
clear
clc
format shortg
tic; 
t0 = tic;

%% Bounding parameters:
start_steps = 0; % Number of steps from faulted branch. 0,1,2...
max_steps = 10; % Max steps allowed.
raiseLimit = 3; % Rise the minimum limit to reduce expansion of layers.

%% Define named indices into bus, gen, branch matrices:
defineNamedIndices;

%% System parameters:
casedata = case24_ieee_rts; %    case118   case2383wp             
mpc = loadcase(casedata);
[baseMVA, bus, gen, branch] = ...
    deal(mpc.baseMVA, mpc.bus, mpc.gen, mpc.branch);
[ref, pv, pq] = bustypes(bus, gen);

%{ 
%% New flows (testing of accuracy):
mpopt = mpoption('model','DC');
[r1, s1] = runpf(mpc, mpopt);
%}

%% Add zero columns to branch for flows if needed:
if size(mpc.branch,2) < QT
mpc.branch = ... 
    [mpc.branch zeros(size(mpc.branch, 1), QT-size(mpc.branch,2))];
end

%% Voltage angles:
Va0 = bus(:,VA) * (pi/180);

%% Build B matrices and phase shift injections:
[B, Bf, Pbusinj, Pfinj] = makeBdc(baseMVA, bus, branch);

%% Compute complex bus power injections (generation - load):
% Adjusted for phase shifters and real shunts.
Pbus = real(makeSbus(baseMVA, bus, gen)) - Pbusinj - bus(:, GS) / baseMVA;

%% "Run" the power flow:
Va = Va0;

%% Update angles for non-reference buses:
Va([pv;pq]) = ...
    B([pv;pq],[pv;pq]) \ (Pbus([pv;pq]) - B([pv;pq],ref) * Va0(ref));

%% Update data matrices with solution:
branch(:, [QF, QT]) = zeros(size(branch, 1), 2);
branch(:, PF) = (Bf * Va + Pfinj) * baseMVA;
branch(:, PT) = -branch(:, PF);
bus(:, VM) = ones(size(bus, 1), 1);
bus(:, VA) = Va * (180/pi);

%% Set x-values:
nl = size(branch,1);
nb = size(bus,1);

%% Find isolated nodes and parallel lines:
IslandAndParallel;

%% Reactance values: 
stat = branch(:,BR_STATUS); %% ones at in-service branches
indices = find(stat == 0);
x = branch(:,BR_X); %% Branch reactances.
x(indices) = NaN;

%% Make adjacent matrix:
bf = branch(:,F_BUS);
bt = branch(:,T_BUS);
adj = sparse(bf, bt, 1, nb, nb);
adj_mat = adj + adj' + speye(nb);

%% LUPQ factorization of B:
[L, U, P, Q] = lu(B([pv;pq],[pv;pq]));

%% Thermal limits:
f0 = branch(:,PF) ./ baseMVA; % f_pq^0
thermal = (branch(:,RATE_B) ./ baseMVA) .* 0.95; % Thermal limits. 
if branch(:,RATE_B) == 0 % If no thermal data present.
    thermalMax = max(abs(f0)) .* 1.3; % 1.3 x the largest flow.
    thermal(:,1) = thermalMax;
    indices = abs(f0) < (100 / baseMVA);
    % Set capacity to 250 for lines < 100 :
    thermal(indices) = 250 / baseMVA;  
    thermal = thermal .* 0.95; % 5% margin due to DC.
    dfmax = thermal; 
else
    dfmax = (thermal - abs(f0));
end

xdfmax = zeros(nl,2);
xdfmax(:,1) = 1:1:nl; 
xdfmax(:,2) = x .* dfmax; % delta f_pq^max*x_pq
sort_xdfmax = sortrows(xdfmax,2); 
maxAngle = thermal .* x; % Define max angle-spread for all branches.

%% Find negative values: 
negative_xdfmax_indices = find(xdfmax(:,2) < 0); 
nNeg = length(negative_xdfmax_indices);

%% The limit is raised:
if nNeg > 0 || raiseLimit > 0
    limit = sort_xdfmax(nNeg+raiseLimit,2);
else
    limit = sort_xdfmax(1,2);
end

%% Branches which must be checked for violation when limit is raised:
if nNeg == 0
    branchCheck = sort_xdfmax(1:1:raiseLimit,1);
else    
    branchCheck = sort_xdfmax(1:1:(nNeg+raiseLimit),1);
end

%% Nodes which must be calculated:
bf = branch(:,F_BUS);
bt = branch(:,T_BUS);
branchCheckNodes = union(bf(branchCheck),bt(branchCheck));

%% Calculate all inverse columns with consecutive indexing: 
timeX = tic;
X = zeros(nb,nb);
for i = 1:nb
    one_node = sparse(i,1,1,nb,1);
    X([pv;pq],i) = Q * (U \ (L \ (P * one_node([pv;pq]))));
end
timeX = toc(timeX);

%% Initializing:
nodes_visited = zeros(nb,nl); % All nodes visited for each contingency. 
numbers_of_nodes_visited = zeros(1,nl); 
angle_spread_log = zeros(max_steps+1,nl); % First row is for n = 0!
success = ones(1,nl); % Keeps track of the success of the boundary method.
count = 0; % Keeps track of contingencies calculated. 

%% Calculate node angle change for lines influenced: 
nodeAngleChange = zeros(nb,1); % Change of voltage-angels.
violation = zeros(nl,nl); % What branches violates flow-limits?
PI = zeros(nl,1); % Performance Indices for contingency ranking.

%% Start the contingency analysis.
for cont = 1:nl 
%% If isolated node:
if ismember(cont,indexIsland)
    angle_spread_log(:,cont) = NaN;
    nodes_visited(:,cont) = NaN; 
    numbers_of_nodes_visited(1,cont) = NaN;
    success(1,cont) = NaN;
    violation(:,cont) = NaN;
    continue; % cont = cont + 1.
end

delta = zeros(nb,1); % Initialize vector of sensitivity factors.
    
branch_nodes = zeros(2,1); 
branch_nodes(1) = bf(cont);
branch_nodes(2) = bt(cont);
node_vec = sparse(branch_nodes,1,1,nb,1);
node_vec_copy_1 = node_vec;

%% Number of start-steps from outaged branch for all contingencies.
n = start_steps; % n keeps track of the steps for rest of the code. 
for i = 1:n
    node_vec = adj_mat * node_vec;
    if n == 1
        node_vec_center = node_vec_copy_1;
    end
    if n > 1 && i == n-1
        node_vec_center = node_vec;
    end
end

all_nodes = bus(node_vec > 0); % All nodes n start_steps from fault branch.
all_nodes_copy = all_nodes; % Needed for second while loop.
if n == 0
boundary_nodes = bus(node_vec > 0);
center_nodes = boundary_nodes;
end
if n > 0
    center_nodes = bus(node_vec_center > 0);
    boundary_nodes = setdiff(all_nodes,center_nodes);
end
node_vec_copy_2 = node_vec; % Needed for the second while loop.

%% The two inverse X columns needed for each contingency:
if bf(cont) ~= ref
X_bf = X(:,bf(cont));
end
if bt(cont) ~= ref
X_bt = X(:,bt(cont));
end

%% Traverse in order to check the angle-spread criterion:
while length(center_nodes) < nb && n <= max_steps

if n == start_steps
    nodes_visited((all_nodes_copy),cont) = all_nodes_copy(:);
    numbers_of_nodes_visited(1,cont) = length(find(nodes_visited(:,cont)));
else
    nodes_visited(all_nodes,cont) = all_nodes(:);
    numbers_of_nodes_visited(1,cont) = length(find(nodes_visited(:,cont)));
end

%% Calculate angle change:
% Calculate sensitivity factors of the center nodes (inside boundary):
if n == start_steps 
    cn = center_nodes; % Shorter name.
    for i = 1:nnz(cn)
        if cn(i) == ref
        delta(cn(i)) = 0;
        elseif bt(cont) == ref
        delta(cn(i)) = (X_bf(cn(i))*x(cont))/(x(cont)-X_bf(bf(cont)));
        elseif  bf(cont) == ref
        delta(cn(i)) = -(X_bt(cn(i))*x(cont))/(x(cont)-X_bt(bt(cont)));
        else
        delta(cn(i)) = (X_bf(cn(i))-X_bt(cn(i)))*x(cont)/ ...
        (x(cont)-(X_bf(bf(cont))+X_bt(bt(cont))-2*X_bt(bf(cont))));
        end  
    end
end

% Calculate sensitivity factors for the nodes due to the "raiseLimit":
if n == start_steps 
    bc = branchCheckNodes; % Shorter name.
    for i = 1:nnz(bc)
        if bc(i) == ref
        delta(bc(i)) = 0;
        elseif bt(cont) == ref
        delta(bc(i)) = (X_bf(bc(i))*x(cont))/(x(cont)-X_bf(bf(cont)));
        elseif  bf(cont) == ref
        delta(bc(i)) = -(X_bt(bc(i))*x(cont))/(x(cont)-X_bt(bt(cont)));
        else
        delta(bc(i)) = (X_bf(bc(i))-X_bt(bc(i)))*x(cont)/ ...
        (x(cont)-(X_bf(bf(cont))+X_bt(bt(cont))-2*X_bt(bf(cont))));
        end  
    end
end

% Calculate sensitivity factors for the boundary nodes:
nBn = nnz(boundary_nodes); % Number of boundary-nodes.
bn = boundary_nodes; % Shorter name of variable.
for i = 1:nBn
    if bn(i) == ref
    delta(bn(i)) = 0;
    elseif bt(cont) == ref
    delta(bn(i)) = (X_bf(bn(i))*x(cont))/(x(cont)-X_bf(bf(cont)));
    elseif bf(cont) == ref
    delta(bn(i)) = -(X_bt(bn(i))*x(cont))/(x(cont)-X_bt(bt(cont)));
    else
    delta(bn(i)) = (X_bf(bn(i))-X_bt(bn(i)))*x(cont)/ ...
    (x(cont)-(X_bf(bf(cont))+X_bt(bt(cont))-2*X_bt(bf(cont))));
    end
end

% Max change for boundary nodes' sensitivity factors: 
delta_bn = delta(bn);
delta_max = max(delta_bn);
delta_min = min(delta_bn);

% Max change angular spread between the boundary nodes:
angleSpread = (delta_max-delta_min) * abs(f0(cont)); 
if angleSpread > 2.0 % Extreme cases, might indicate "islanding" of node. 
    angleSpread = NaN;
    success(1,cont) = 0;
end
angle_spread_log(n+1,cont) = angleSpread;

% Check the criterion: 
if angleSpread <= limit 
    break;
elseif length(all_nodes) == nb % angleSpread <= limit not possible.
    success(cont) = 0;
    break;
elseif n == max_steps % Criterion not fulfiled for max_steps.
    success(cont) = 0;
    break;
else
    
%% Expand boundary further:
n = n + 1; % Number of steps from fault branch.

if n == start_steps + 1
node_vec = node_vec_copy_2;
node_vec_center = node_vec_copy_2;
else
node_vec_center = node_vec;
end

node_vec = adj_mat * node_vec;

all_nodes = bus(node_vec > 0); % All nodes k start_steps from fault branch.
center_nodes = bus(node_vec_center > 0);
boundary_nodes = setdiff(all_nodes,center_nodes);
end 
end 

%% Calculate the new flows and PI: 
nodeAngleChange = f0(cont) .* delta;
newAngle = Va + nodeAngleChange;  
angleSpread = newAngle(bf) - newAngle(bt);

% Find overloaded branches:
findViolation = find(abs(angleSpread) > maxAngle);
violation(findViolation,cont) = findViolation; 
for k = 1:nnz(findViolation)  
    
    if findViolation(k) == cont % Are out of service. 
        continue;
    end
            
    tempPI = ...
        (angleSpread(findViolation(k)) / maxAngle(findViolation(k)))^2 ; 
    PI(cont) = PI(cont) + tempPI;
    
    %if PI(cont) > 10^5 % If huge overload. 
    %PI(cont) = 10^5; % Max PI value.
    %break;
    %end  
    
end

count = count + 1
end

%% Find the highest PIs and sort them:
ind = find(PI);
ni = length(ind);
result = zeros(ni,2);
for a = 1:ni
    result(a,1) = ind(a);
    result(a,2) = PI(ind(a)); 
end
resultPI = sortrows(result,-2);
time = toc;

%% Success & total time consumption: 
not_success = find(success == 0);
totalTime = toc(t0);

%% Output:
negative_xdfmax_indices
limit
numbers_of_nodes_visited
not_success
resultPI
timeX
totalTime
whos X

%{   
% New flow for testing accuracy & additional output:
branchOut = 3; 
mpc.branch(branchOut,BR_STATUS) = 0;
[r2,s2] = runpf(mpc, mpopt);

nodeAngleChange = nodeAngleChange .* 180/pi
angleUpdated = r1.bus(:,VA) + nodeAngleChange 
violation_test = nonzeros(violation(:,branchOut))
numbers_violation_branches = nnz(violation_test)
nodes_visited_test = nonzeros(nodes_visited(:,branchOut))
angle_spread_log_test = angle_spread_log(:,branchOut)
%}
%