%% ~*Fast Decoupled Load Flow Contingency Analysis With Compensation*~
clear
clc
format shortg

%% Start stopp watch:
tic; % Start.
t0 = tic;

%% *Control Parameters*
tol = 0.01; % 1% seems like a good trade-off between speed and accuracy.
max_it = 10; 

% Define named indices into bus, gen, branch matrices:
defineNamedIndices;

%% Power system data:
mpc = loadcase(case2383wp);   %      case24_ieee_rts     case3120sp
[bus,gen,branch] = deal(mpc.bus,mpc.gen,mpc.branch);
[ref, pv, pq] = bustypes(bus,gen); % Make lists of node types.  
nb = length(bus(:,1));
nl = length(branch(:,1));

%% Find isolated nodes and parallel lines:
IslandAndParallel;

%% Reference Flow:
mpopt = mpoption('pf.alg','NR', 'pf.tol',10^-5 );
[r1,s1] = runpf(mpc, mpopt);
[bus,gen,branch,baseMVA] = deal(r1.bus,r1.gen,r1.branch,r1.baseMVA);

%% initial state:   
on = find(gen(:, GEN_STATUS) > 0); % Which generators are on?
gbus = gen(on, GEN_BUS); % What buses are they at?

V0  = bus(:, VM) .* exp(sqrt(-1) * pi/180 * bus(:, VA));
vcb = ones(size(V0)); % Create mask of voltage-controlled buses.
vcb(pq) = 0;          % Exclude PQ-buses.
k = find(vcb(gbus));  % In-service gens at v-c buses.
V0(gbus(k)) = gen(on(k), VG) ./ abs(V0(gbus(k))).* V0(gbus(k));
Va0 = angle(V0); % Initial voltage-angle magnitude.
Vm0 = abs(V0); % Initial voltage magnitude.

%% Build admittance matrix:
[Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch);

%% Series admittance, Ys:
stat = branch(:,BR_STATUS); % Ones at in-service branches.
Ys = stat ./ (branch(:,BR_R) + 1j * branch(:, BR_X)); 

%% Compute complex bus power injections (generation - load):
Sbus = makeSbus(baseMVA, bus, gen);

%% B-matrices for BX-version:
[Bp0, Bpp0] = makeB(baseMVA,bus,branch,'FDBX');
% Reduce B-matrices:
Bp = Bp0([pv; pq],[pv; pq]);
Bpp = Bpp0(pq,pq);

%% factor B matrices
[Lp, Up, Pp, Qp] = lu(Bp);
[Lpp, Upp, Ppp, Qpp] = lu(Bpp);

%% Calculate PTDFs & LODFs:
timeLODFsStart = tic;
PTDFs = makePTDF(baseMVA,bus,branch);
LODFs = makeLODF(branch,PTDFs);
clear PTDFs
timeLODFs = toc(timeLODFsStart);

%% Initializing:
% Define power flow matrices:
Pflow = branch(:,PF);
PflowThermal = branch(:,RATE_B) * 0.95; % Add 5% margin due to DC.
PflowNew = zeros(nl,nl);
PflowViolation = zeros(nl,3);
% Define voltage matrices:
voltageViolationLow = zeros(nb,3);
voltageViolationHigh = zeros(nb,3);
resVm = ones(nb,nl);
resVa = zeros(nb,nl);
% List of iterations count:
iterations = zeros(nb,1); 
count = 0; % Number of cases tested. 

%% Start the contingency analysis:  
for cont = 1:nl 
%% If isolated node:
if ismember(cont,indexIsland)
    resVm(:,cont) = NaN;
    resVa(:,cont) = NaN;
    PflowNew(:,cont) = NaN;
    iterations(cont) = NaN;
    continue; % Contiune for loop with x = x + 1.
end

%% Use pre-contingency values: 
Va = Va0;
Vm = Vm0; 

% Branch nodes:
fBus = branch(cont,F_BUS);
tBus = branch(cont,T_BUS);

%% Calculate compensation terms: 
M = sparse([fBus tBus],1,[1 -1],nb,1);
Mp = M([pv;pq]);
Mq = M(pq);

% If parallel lines:
if ismember(cont,indexParallel)
    dyp = Bp0(fBus,tBus) / 2;
    dyq = Bpp0(fBus,tBus) / 2;
else
    dyp = Bp0(fBus,tBus);
    dyq = Bpp0(fBus,tBus);  
end

invBp_Mp = Qp * (Up \(Lp \(Pp * Mp ))); % Pre-calculate Mp*H^-1.
invBpp_Mq = Qpp * (Upp \(Lpp \(Ppp *Mq )));
zp = Mp.' * invBp_Mp; %% M^t*H^-1*M
zq = Mq.' * invBpp_Mq; %% M^t*H^-1*M
cp = 1/(1/dyp + zp);
cq = 1/(1/dyq + zq);

%% Update Ybus:
Ybus2 = Ybus + M*-Ys(cont)*M';

%% Initial mismatch
mis = (V0 .* conj(Ybus2*V0) - Sbus) ./ Vm;
P = real(mis([pv;pq]));
Q = imag(mis(pq));

%% Iniatilizing:
converged = 0;
y = 0;

%% Run FDLF with compensation:
while (~converged && y < max_it)
y = y + 1;

%% ----- do P iteration, update Va -----
dVa = - Qp * (Up \ (Lp \ (Pp * P)));
dVaComp = cp * invBp_Mp * Mp.' * dVa;
Va([pv;pq]) = Va([pv;pq]) + dVa - dVaComp;

%% ----- do Q iteration, update Vm -----
dVm = - Qpp * (Upp \ (Lpp \ (Ppp * Q)));
dVmComp = cq * invBpp_Mq * Mq.' * dVm;      
Vm(pq) = Vm(pq) + dVm - dVmComp;
V = Vm .* exp(1j * Va);

%% Evalute mismatches:
mis = (V .*conj(Ybus2 * V) - Sbus) ./ Vm;
P = real(mis([pv;pq]));
Q = imag(mis(pq));

%% check tolerance
normP = norm(P, inf);
normQ = norm(Q, inf);
if normP <= tol && normQ <= tol
    converged = 1;
    resVm(:,cont) = abs(V(:,1));
    resVa(:,cont) = Va(:,1);
    iterations(cont) = y;
    break;
end
if y == max_it
    iterations(cont) = max_it;
    break;
end
end

%% Use LODFs to estimate new active power flow:   
PflowNew(:,cont) = Pflow + LODFs(:,cont) .* Pflow(cont);

%% Find the maximal flow overshooting:
% Find the highest violation.
[F1,I1] = min(PflowThermal - abs(PflowNew(:,cont))); 
if F1 < 0 % Any flow-violations?
    PflowViolation(cont,1) = cont; % The contingency which cause it.
    % What line has the biggest overshooting (absolute value):
    PflowViolation(cont,2) = I1;  
    % Find the MW-value:
    PflowViolation(cont,3) = abs(F1);
end

%% Find the minimal voltage-magnitude:
[V2,I2] = min(resVm(:,cont));
if V2 < 0.90 % Violation of the limit?
   voltageViolationLow(cont,1) = cont; % The contingency which cause it.
   voltageViolationLow(cont,2) = I2; % What node has the least voltage?
   voltageViolationLow(cont,3) = V2; % What is the the value?
end

%% Find the maximal voltage-magnitude:
[V3,I3] = max(resVm(:,cont));
if V3 > 1.10
    voltageViolationHigh(cont,1) = cont; % The contingency which cause it. 
    voltageViolationHigh(cont,2) = I3; % What node has the largest voltage?
    voltageViolationHigh(cont,3) = V3; % What is the value?
end

count = count + 1
end
totalTime = toc;

%% Output section:

%}  %}
% Additional output. Not to be used for big systems. 
resVm;
resVa = resVa * 180/pi;
PflowNew;

countIsland
countParallel
iterations

% Display the violations:
I4 = nonzeros(PflowViolation(:,1));
PflowViolation = PflowViolation(I4,:)
I5 = nonzeros(voltageViolationLow(:,1));
voltageViolationLow = voltageViolationLow(I5,:)
I6 = nonzeros(voltageViolationHigh(:,1));
voltageViolationHigh = voltageViolationHigh(I6,:)

timeLODFs
totalTime
whos LODFs

% Activate to test the accuracy of the code: 
%{ 
lineOut = 28;
mpc.branch(lineOut,BR_STATUS) = 0;
[r2,s2] = runpf(mpc, mpopt);
flow = PflowNew(:,lineOut)
voltageMagnitude = resVm(:,lineOut)
voltageAngle = resVa(:,lineOut)
%}

%