%% ~*Double DC Contingency Analysis With Compensation*~
clear
clc
format shortg
tic

mpc = loadcase(case24_ieee_rts); %  case118  case2383wp

% Define named indices into bus, gen, branch matrices:
defineNamedIndices;

% Split up:
[baseMVA, bus, gen, branch] = ...
    deal(mpc.baseMVA, mpc.bus, mpc.gen, mpc.branch);

nb = length(bus(:,1)); % Number of buses. Used in "IslandAndParallel".
nl = length(branch(:,1)); 

%% Find isolated nodes and parallel lines:
IslandAndParallel;

% Create reference:
mpopt = mpoption('pf.alg','NR', 'pf.tol',10^-5);
[r1, s1] = runpf(mpc, mpopt);
Va0 = r1.bus(:,VA) * pi/180; % Initial voltage-angles.

%% Thermal limits:
f0 = r1.branch(:,PF) ./ baseMVA; % Initial flow.
thermal = zeros(nl,1);
if branch(:,RATE_B) == 0 % If no thermal data present.
    thermalMax = max(abs(f0)) .* 1.3; % 1.3 x the largest flow.
    thermal(:,1) = thermalMax;
    indices = abs(f0) < (100 / baseMVA);
    % Set capacity to 250 for lines < 100 :
    thermal(indices) = 250 / baseMVA; 
    thermal = thermal .* 0.95; % 5% margin due to DC.
else
    thermal = (branch(:,RATE_B) ./ baseMVA) .* 0.95; % Thermal limits.   
end

%% Determine angle spread limits for each line:
x = branch(:,BR_X); % Branch reactances.
maxAngle = thermal .* x; % Define max angle spread for each line.

% Build B-matrix:
[Bbus1,~,~,~] = makeBdc(baseMVA, bus, branch);

% Builds index lists for each type of bus (REF, PV, PQ):
[ref, pv, pq] = bustypes(bus, gen);

% Factorization:
Bt = Bbus1([pv; pq],[pv; pq]);
[L0, U0, P0, Q0] = lu(Bt);

nb = length(bus(:,1));
nl = length(branch(:,1));

bf = branch(:,F_BUS);
bt = branch(:,T_BUS);

resVa1 = zeros(nb,nl);
resVa2 = zeros(nb,1);
PI = zeros(nl,nl);
count = 0;

%% Double compensation starts:
for b1 = 1:nl

% If isolated node:
if ismember(b1,indexIsland)
    continue;
end
    
bf1 = branch(b1,F_BUS);
bt1 = branch(b1,T_BUS);

M1 = sparse([bf1 bt1],[ 1 1],[1 -1],nb,1);
Mindex = M1([pv;pq]);

invBbusM = Q0 * (U0 \ (L0 \ (P0 * Mindex )));
z = Mindex' * invBbusM;

if ismember(b1,indexParallel) % If parallel lines:
    dy = Bbus1(bf1,bt1) / 2;
else
    dy = Bbus1(bf1,bt1);    
end

c = 1 / ((1/dy) + z);

deltaVa = invBbusM * c * Mindex' * Va0([pv;pq]); 
VaComp = Va0([pv;pq]) - deltaVa;
resVa1([pv;pq],b1) = VaComp;

Bbus2 = Bbus1 + M1*dy*M1';

% Factorization for second loop:
Bt = Bbus2([pv; pq],[pv; pq]);
[L, U, P, Q] = lu(Bt);

%% Second stage:
for b2 = 1:nl

if b2 <= b1
    continue % Make sure unique combinations. 
end

% If isolated node:
if ismember(b2,indexIsland)
    continue;
end

%% Compensation:
bf2 = branch(b2,F_BUS);
bt2 = branch(b2,T_BUS);

M2 = sparse([bf2 bt2],[ 1 1],[1 -1],nb,1);
Mindex = M2([pv;pq]);

invBbusM = Q * (U \ (L \ (P * Mindex )));
z = Mindex' * invBbusM;

% If parallel lines:
if ~ismember(b1,indexParallel) && ismember(b2,indexParallel) 
    dy = Bbus2(bf2,bt2) / 2;
elseif ismember(b1,indexParallel) && ismember(b2,indexParallel) && b1 ~= b2 
    dy = Bbus2(bf2,bt2) / 2;
else
    dy = Bbus2(bf2,bt2);    
end

c = 1 / ((1/dy) + z);

deltaVa = invBbusM * c * Mindex' * resVa1([pv;pq],b1); 
VaComp = resVa1([pv;pq],b1) - deltaVa;
resVa2([pv;pq]) = VaComp;

angleSpread = resVa2(bf) - resVa2(bt);

violation = find(abs(angleSpread) > maxAngle); % Find overloaded branches.
for k = 1:nnz(violation)    
    if violation(k) == b1 || violation(k) == b2 % Are out of service. 
        continue;
    end 
    
    tempPI = (angleSpread(violation(k)) / maxAngle(violation(k)))^2 ;
    PI(b1,b2) = PI(b1,b2) + tempPI;
    
    %if PI(b1,b2) > 10^5 % If huge overload. 
    %PI(b1,b2) = 10^5; % Max PI value.
    %break;
    %end       
end
count = count + 1
end
end

%% Find the highest PIs and sort them:
[row,col] = find(PI);
nr = length(row);
result = zeros(nr,3);
for a = 1:nr
    result(a,1) = row(a);
    result(a,2) = col(a);
    result(a,3) = PI(row(a),col(a)); 
end
resultPI = sortrows(result,-3);
time = toc;

%{
violation_branches = violation
violation_value = angleSpread(violation)
%}

%% Output:
indexIsland
indexParallel
resultPI
count
time

%{ 
% New flows:
mpopt = mpoption('pf.alg','NR', 'pf.tol',10^-5);
mpc.branch([7 24],BR_STATUS) = 0;
[r2, s2] = runpf(mpc, mpopt);

resVa2 = resVa2 .* 180/pi
%}

 






