%% ~*Single DC Contingency Analysis - Generator Outage*~
clear
clc
format shortg
tic

% Define named indices into bus, gen, branch matrices:
defineNamedIndices;

%% Power system data:
mpc = loadcase(case2383wp);  %   case118  case24_ieee_rts
[bus,gen,branch,baseMVA] = deal(mpc.bus,mpc.gen,mpc.branch,mpc.baseMVA);
% Builds index lists for each type of bus (REF, PV, PQ):
[ref, pv, pq] = bustypes(bus,gen);   

nb = length(bus(:,1)); % Number of buses.
ng = length(gen(:,1)); % Number of generators.
nl = length(branch(:,1)); % Number of lines.

%% Create reference (testing):
mpopt = mpoption('model','DC');
[r1, s1] = runpf(mpc, mpopt);

%% Thermal limits:
thermal = zeros(nl,1);
if branch(:,RATE_B) == 0
    % 1.3 x the largest flow:
    thermalMax = max(abs(r1.branch(:,PF))) .* 1.3; 
    thermal(:,1) = thermalMax;
    indices = abs(r1.branch(:,PF)) < 100;
    thermal(indices) = 250; % Set capacity to 250 for lines < 100  
else
    thermal = (branch(:,RATE_B)) .* 0.95 ;  % 5% margin due to DC.
end

%% Build B matrices and phase shift injections:
[B,~,~,~] = makeBdc(baseMVA, bus, branch);

%% LUPQ factorization of B:
[L,U,P,Q] = lu(B([pv;pq],[pv;pq]));

%% Calculate the needed inverse columns with consecutive indexing: 
timeX = tic;
genList = gen(:,GEN_BUS);
genListUnique = unique(genList); % Might be > 1 generator at each node.
nglu = nnz(genListUnique);
X = zeros(nb,nb);
for i = 1:nglu
    one_node = sparse(genListUnique(i),1,1,nb,1);
    X([pv;pq],genListUnique(i)) = Q * (U \ (L \ (P * one_node([pv;pq]))));
end
timeX = toc(timeX);

%% Branch vectors:
bf = branch(:,F_BUS);
bt = branch(:,T_BUS);

%% Reactance values: 
stat = branch(:,BR_STATUS); %% Ones at in-service branches.
indices = find(stat == 0);
x = branch(:,BR_X); %% Branch reactances.
x(indices) = NaN;

%% Generator outage analysis:
newFlow = zeros(nl,1); % New flows for each generator outage.
count = 0; % The number of generator outages tested.
PI = zeros(ng,1); % Performance Index.
for i =  1:ng
    j = gen(i,GEN_BUS);   
    
    if j == ref
        continue; % Slack bus.
    end
    
    temp = X(:,j);
    PTDF = (temp(bf) - temp(bt)) ./ x;  
    newFlow = r1.branch(:,PF) + PTDF .* -r1.gen(i,PG);
    
    indices = find(abs(newFlow(:)) > thermal);
    ni = nnz(indices);
    for k = 1:ni
        PItemp = (abs(newFlow(indices(k))) / thermal(indices(k)))^2;
        PI(i) = PI(i) + PItemp;
    end
    
count = count + 1
end

%% Find the highest PI and sort them:
I = find(PI);
nI = nnz(I);
result = zeros(nI,3);
for a = 1:nI
    result(a,1) = I(a);
    result(a,2) = gen(I(a),GEN_BUS);
    result(a,3) = PI(I(a)); 
end
resultPI = sortrows(result,-3);
totalTime = toc;

genList 
resultPI
timeX
totalTime 
whos X

%{
% New flows (testing):
generator = 1;
mpc.gen(generator,GEN_STATUS) = 0;
[r2, s2] = runpf(mpc, mpopt);
newFlowOutput = newFlow
%}














