%% ~*Single DC Contingency Analysis With Compensation*~
clear
clc
format shortg
tic

mpc = loadcase(case2383wp); % case118  case24_ieee_rts  

% Define named indices into bus, gen, branch matrices:
defineNamedIndices;

% Split up:
[baseMVA, bus, gen, branch] = deal(mpc.baseMVA, mpc.bus, mpc.gen, mpc.branch);

nb = length(bus(:,1)); % Number of buses. Used in "IslandAndParallel".
nl = length(branch(:,1)); 

%% Find isolated nodes and parallel lines:
IslandAndParallel;

% Create reference:
mpopt = mpoption('pf.alg','NR', 'pf.tol',10^-5);
[r1, s1] = runpf(mpc, mpopt);
Va0 = r1.bus(:,VA) * pi/180; % Initial voltage-angles.

%% Thermal limits:
f0 = r1.branch(:,PF) ./ baseMVA; % Initial flow.
thermal = zeros(nl,1);
if branch(:,RATE_B) == 0
    thermalMax = max(abs(f0)) * 1.3; % 1.3 x the largest flow.
    thermal(:) = thermalMax;
    indices = abs(f0) < (100 / baseMVA);
    % Set capacity to 250 for lines < 100:
    thermal(indices) = 250 / baseMVA; 
    thermal = thermal .* 0.95; % 5% margin due to DC.
else
    thermal = (branch(:,RATE_B) ./ baseMVA) .* 0.95; % Thermal limits.   
end

%% Determine angle spread limits for each line:
x = branch(:,BR_X); % Branch reactances.
maxAngle = thermal .* x; % Set max angle-spread for each line.

% Build B-matrix:
[Bbus1,~,~,~] = makeBdc(baseMVA, bus, branch);

% Builds index lists for each type of bus (REF, PV, PQ):
[ref, pv, pq] = bustypes(bus, gen);

% Factorization:
Bt = Bbus1([pv; pq],[pv; pq]);
[L0, U0, P0, Q0] = lu(Bt);

nb = length(bus(:,1));
nl = length(branch(:,1));

bf = branch(:,F_BUS);
bt = branch(:,T_BUS);

resVa = zeros(nb,1);
PI = zeros(nl,1);
count = 0;

%% Single DC compensation starts:
for b1 = 1:nl

% If isolated node:
if ismember(b1,indexIsland)
    continue; % b1 = b1 + 1 
end
    
bf1 = branch(b1,F_BUS);
bt1 = branch(b1,T_BUS);

M1 = sparse([bf1 bt1],[1 1],[1 -1],nb,1);
Mindex = M1([pv;pq]);

invBbusM = Q0 * (U0 \ (L0 \ (P0 * Mindex )));
z = Mindex' * invBbusM;

if ismember(b1,indexParallel) % If parallel lines:
    dy = Bbus1(bf1,bt1) / 2;
else
    dy = Bbus1(bf1,bt1);    
end

c = 1 / ((1/dy) + z);

deltaVa = invBbusM * c * Mindex' * Va0([pv;pq]); 
VaComp = Va0([pv;pq]) - deltaVa;
resVa([pv;pq]) = VaComp;

angleSpread = resVa(bf) - resVa(bt); % Angle-spread of all branches. 

violation = find(abs(angleSpread) > maxAngle); % Find overloaded branches.
for k = 1:nnz(violation)    
    
    if violation(k) == b1 % Is out of service. 
        continue;
    end 
    
    tempPI = (angleSpread(violation(k)) / maxAngle(violation(k)))^2 ; 
        
    %if PI(b1) > 10^3 % If huge overload. 
    %PI(b1) = 10^3; % Max PI value.
    %break;
    %end
    
    PI(b1) = PI(b1) + tempPI;  
    
end

count = count + 1
end

[I] = find(PI);
ni = length(I);
resultPI = zeros(ni,2);
for a = 1:ni
    resultPI(a,1) = I(a);
    resultPI(a,2) = PI(I(a)); 
end
resultPI = sortrows(resultPI,-2);
totalTime = toc;

%% Output:
resultPI
count
totalTime

%{  
% New flows (testing):
mpopt = mpoption('model','DC');
%mpopt = mpoption('pf.alg','NR', 'pf.tol',10^-5);
branchOut = 7;
mpc.branch(branchOut,BR_STATUS) = 0;
[r2, s2] = runpf(mpc, mpopt);
resVa = resVa .* 180/pi
% Comment: When the reference flow is AC, 
% the results will be a bit different when running a DC algorithm.
% AC gives a more accurate inital voltage-angles. 
%}








